#!/usr/bin/perl -w
#
# file    : create-static-html-from-mod_perl.pl 
# changed : 2005-02-01
# author  : Bas Couwenberg <sebastic@xs4all.nl>
#

use strict;
use LWP::UserAgent;
use File::Basename;
use Date::Calc;
use Getopt::Mixed "nextOption";

my ($year, $month, $day, $hour, $min, $sec) = Date::Calc::Today_and_Now();

$month = '0'.$month if(length($month) == 1);
$day   = '0'.$day   if(length($day)   == 1);
$hour  = '0'.$hour  if(length($hour)  == 1);
$min   = '0'.$min   if(length($min)   == 1);
$sec   = '0'.$sec   if(length($sec)   == 1);

my $timestamp = "$year-$month-$day $hour:$min:$sec";


my $self    = 'create-static-html-from-mod_perl.pl';
my $version = '0.1';
my $wwwroot = '/var/www/folding/';

# default configuration
my %cfg = ( 
		self			=> $self,
		version			=> $version,
		url			=> "http://example.com/mod_perl/index.pl",
		wwwroot			=> $wwwroot,
		useragent		=> "$self/$version",
		verbose 		=> 0
	);

Getopt::Mixed::init("url=s wwwroot=s useragent=s verbose help");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	
	$cfg{wwwroot}   = $value if($option eq 'wwwroot');
	$cfg{url}	= $value if($option eq 'url');
	$cfg{useragent} = $value if($option eq 'useragent');
	$cfg{verbose}   = 1      if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

# check parameters
if(!$cfg{useragent} || $cfg{useragent} eq '' || $cfg{useragent} =~ /^\s+$/) {
	print "Error: Invalid User-Agent! ($cfg{useragent})\n";
	exit;
}
elsif(!$cfg{url} || $cfg{url} =~ /^\s*$/ || &invalid_url($cfg{url})) {
	print "Error: Invalid URL! ($cfg{url})\n";
	exit;
}
elsif(!$cfg{wwwroot} || $cfg{wwwroot} =~ /^\s*$/ || !-w $cfg{wwwroot}) {
	print "Error: ";
	if(!-w $cfg{wwwroot}) {
		print "Cannot write target directory! (wwwroot: $cfg{wwwroot})\n";
	}
	else {
		print "Invalid target directory! (wwwroot: $cfg{wwwroot})\n";
	}
}
		

my $ua = LWP::UserAgent->new;
   $ua->agent($cfg{useragent});

# Get the index page, parse the menu component to fetch the urls to the other pages
if(my $indexpage = fetch_url($ua, $cfg{url})) {
	my $indexfile = $cfg{wwwroot}.'index.html';

	# grap stylesheet name, to create and url and fetch it
	print "\nLooking up stylesheet url\n" if($cfg{verbose});
	
	# <link rel="stylesheet" href="stylesheet.css">
	if($indexpage =~ /<link rel="stylesheet" href="(.*?\.css)">/) {
		my $cssfile = $1;
		my $cssurl  = dirname($cfg{url}).'/'.$cssfile;
		
		print "Found css url ($cssurl)\n" if($cfg{verbose});
		if(my $csspage = fetch_url($ua, $cssurl)) {
			$cssfile = $cfg{wwwroot}.$cssfile;

			# save css file
			save_file($cssfile, $csspage);
		}
		else {
			print "Error: Failed to retrieve stylesheet! ($cssurl)\n";
		}
	}
	else {
		print "Error: Cannot find stylesheet url in index file, please copy manually.\n\n";
	}

	
	# grap menu urls, to fetch them
	print "\nLooking up menu urls\n" if($cfg{verbose});

	if($indexpage =~ /<!-- start menu -->/ && $indexpage =~ /<!-- end menu -->/) {
		#<a href='?target=home.plc' class='selected'>Home</a><br>
		my @url = ($indexpage =~ /<a href='(\?target=.*?\.plc)'/g);
		
		my %urls;
		foreach my $url (@url) {
			#$urls{$url} = ''   if(!$urls{$url});
			#$urls{$url} = $url if($urls{$url} ne $url);
		
			print "\nFound URL in menu: $url\n" if($cfg{verbose});

			if($url !~ /home\.plc$/) {
				(my $pagename = $url) =~ s/^.*?target=(.*?)\.plc.*?$/$1/;

				my $url = $cfg{url}.$url;
				if(my $page = fetch_url($ua, $url)) {
					my $pagefile = $cfg{wwwroot}.$pagename.'.html';

					if($page =~ /<a href='(\?target=gpl\.plc)'/) {
						print "\nFound GPL URL in page\n" if($cfg{verbose});
						
						my $url = $cfg{url}.$1;
						
						(my $pagename = $url) =~ s/^.*?target=(.*?)\.plc.*?$/$1/;
					
						if(my $gplpage = fetch_url($ua, $url)) {
							my $gplpagefile = $cfg{wwwroot}.$pagename.'.html';

							# replace urls with links to static pages
							$gplpage = replace_url($gplpage);
							
							# insert update time stamp
							$gplpage = insert_lastupdate($gplpage, $timestamp);
							
							# save page
							save_file($gplpagefile, $gplpage);
						}
						else {
							print "Error: Failed to retrieve page! ($url)\n";
						}
					}

					#<a href='?target=usernames.plc&amp;username=[DPC]_Fatal_Error_Group0smoking2000&amp;team=92'
					my @url = ($page =~ /<a href='(\?target=$pagename\.plc\&amp\;.*?)'/g);
					
					foreach my $url (@url) {
						$urls{$url} = ''   if(!$urls{$url});
						$urls{$url} = $url if($urls{$url} ne $url);
					}

					# replace urls with links to static pages
					$page = replace_url($page);
					
					# insert update time stamp
					$page = insert_lastupdate($page, $timestamp);
					
					# save page
					save_file($pagefile, $page);
				}
				else {
					print "Error: Failed to retrieve page! ($url)\n";
				}
			}
		}

		# fetch the pages linked to in the grapped pages
		foreach my $url (sort keys %urls) {
			print "\nFound URL in page: $url\n" if($cfg{verbose});
			
			(my $pagename    = $url) =~ s/^.*?target=(.*?)\.plc.*?$/$1/;
			(my $subpagename = $url) =~ s/^.*?target=.*?\.plc&amp;(.*?)$/$1/;

			(my $url = $cfg{url}.$url) =~ s/&amp;/&/g;
			if(my $subpage = fetch_url($ua, $url)) {
				my $subpagefile  = $cfg{wwwroot}.$pagename;
				foreach my $param (split /&amp;/, $subpagename) {
					my ($name, $value) = split /=/, $param;
					
					$subpagefile .= '-'.$value;
				}
				$subpagefile .= '.html';
				
				# replace urls with links to static pages
				$subpage = replace_url($subpage);
				
				# insert update time stamp
				$subpage = insert_lastupdate($subpage, $timestamp);
				
				# save page
				save_file($subpagefile, $subpage);
			}
			else {
				print "Error: Failed to retrieve page! ($url)\n";
			}
		}
	}
	else {
		print "Error: Cannot find menu content in index file!\n";
	}
	 
	# replace urls with links to static pages
	$indexpage = replace_url($indexpage);
	
	# insert update time stamp
	$indexpage = insert_lastupdate($indexpage, $timestamp);
	
	# save index file
	save_file($indexfile, $indexpage);
}
else {
	print "Error: Failed to retrieve index file! ($cfg{url})\n";
}

################[ Subroutines ]################

sub insert_lastupdate {
	my $page      = $_[0];
	my $timestamp = $_[1];

	if($page =~ s#(</table>\n<!-- end main -->)#<tr><td class='right-border'><sub>Last Update: $timestamp</sub></td><td></td></tr>\n$1#) {}

	return $page;
}

sub replace_url {
	my $page = $_[0];
	
	print "Replacing dynamic urls with static ones.\n" if($cfg{verbose});
	
	if($page =~ s/\?target=home.plc/index.html/) {}
	if($page =~ s/\?target=(.*?).plc'/$1\.html'/g) {}
	if($page =~ s/\?target=(.*?).plc&amp;.*?=(.*?)&amp;.*?=(.*?)'/$1-$2-$3\.html'/g) {}
	if($page =~ s/\?target=(.*?).plc&amp;.*?=(.*?)'/$1-$2\.html'/g) {}
	if($page =~ s/\?target=(.*?).plc(#.*?)'/$1\.html$2'/g) {}

	return $page;
}

sub save_file {
	my $filename = $_[0];
	my $content  = $_[1];
	
	print "Saving file to disc ($filename)\n" if($cfg{verbose});
	if(!-e $filename || -w $filename) {
		open(F, "> $filename") || die "Cannot open file ($!)";
		print F $content;
		close F;

		return 1;
	}
	else {
		print "Error: Cannot write file! ($filename)\n";
		
		return undef;
	}
}

sub fetch_url {
	my $ua  = $_[0];
	my $url = $_[1];

	my $req = HTTP::Request->new(GET => $url);

	print "Retrieving url ($url)\n" if($cfg{verbose});
	my $res = $ua->request($req);
	if($res->is_success) {
		print "Successfully got url ($url)\n" if($cfg{verbose});

		return $res->content;
	}
	else {
		print "Error: Failed to retrieve url! ($url)\n";

		return undef;
	}
}

sub invalid_url {
	my $url = $_[0];
	
	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	
	my $req = HTTP::Request->new(HEAD => $url);
	
	my $res = $ua->request($req);
	if($res->is_success()) {
		return 0;
	}
	elsif($res->is_error()) {
		return 1;
	}
	else {
		return 1;
	}
}

sub print_usage {
	my $verbose = $cfg{verbose} ? 'on' : 'off';

	print "$cfg{self} v$cfg{version}\n\n";
	print "Usage: ".basename($0)." [OPTIONS]\n\n";
	print "Options:\n";
	print "--url <url>          URL to fci mod_perl site       (default: $cfg{'url'})\n";
	print "--wwwroot <dir>      Directory to store html files  (default: $cfg{'wwwroot'})\n";
	print "--useragent <name>   User-Agent string              (default: $cfg{'useragent'})\n";
	print "--verbose            Verbose (debug) output         (default: $verbose)\n";
	print "--help               Print this information\n";
}
